package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.ArrayInitializer;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.NameReference;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.ElementValuePair;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;

public class MemberValuePair extends ASTNode {
  public char[] name;
  
  public Expression value;
  
  public MethodBinding binding;
  
  public ElementValuePair compilerElementPair = null;
  
  public MemberValuePair(char[] token, int sourceStart, int sourceEnd, Expression value) {
    this.name = token;
    this.sourceStart = sourceStart;
    this.sourceEnd = sourceEnd;
    this.value = value;
    if (value instanceof ArrayInitializer)
      value.bits |= 0x1; 
  }
  
  public StringBuffer print(int indent, StringBuffer output) {
    output
      .append(this.name)
      .append(" = ");
    this.value.print(0, output);
    return output;
  }
  
  public void resolveTypeExpecting(BlockScope scope, TypeBinding requiredType) {
    TypeBinding valueType;
    if (this.value == null) {
      this.compilerElementPair = new ElementValuePair(this.name, this.value, this.binding);
      return;
    } 
    if (requiredType == null) {
      if (this.value instanceof ArrayInitializer) {
        this.value.resolveTypeExpecting(scope, null);
      } else {
        this.value.resolveType(scope);
      } 
      this.compilerElementPair = new ElementValuePair(this.name, this.value, this.binding);
      return;
    } 
    this.value.setExpectedType(requiredType);
    if (this.value instanceof ArrayInitializer) {
      ArrayInitializer initializer = (ArrayInitializer)this.value;
      valueType = initializer.resolveTypeExpecting(scope, this.binding.returnType);
    } else if (this.value instanceof org.eclipse.jdt.internal.compiler.ast.ArrayAllocationExpression) {
      scope.problemReporter().annotationValueMustBeArrayInitializer((TypeBinding)this.binding.declaringClass, this.name, this.value);
      this.value.resolveType(scope);
      valueType = null;
    } else {
      valueType = this.value.resolveType(scope);
      Object object1 = new Object(this);
      this.value.traverse((ASTVisitor)object1, scope);
    } 
    this.compilerElementPair = new ElementValuePair(this.name, this.value, this.binding);
    if (valueType == null)
      return; 
    TypeBinding leafType = requiredType.leafComponentType();
    boolean[] shouldExit = new boolean[1];
    Object object = new Object(this, valueType, requiredType, leafType, scope, shouldExit);
    if (!scope.deferCheck((Runnable)object)) {
      object.run();
      if (shouldExit[0])
        return; 
    } 
    switch ((leafType.erasure()).id) {
      case 2:
      case 3:
      case 4:
      case 5:
      case 7:
      case 8:
      case 9:
      case 10:
      case 11:
        if (this.value instanceof ArrayInitializer) {
          ArrayInitializer initializer = (ArrayInitializer)this.value;
          Expression[] expressions = initializer.expressions;
          if (expressions != null)
            for (int i = 0, max = expressions.length; i < max; i++) {
              Expression expression = expressions[i];
              if (expression.resolvedType != null && 
                expression.constant == Constant.NotAConstant)
                scope.problemReporter().annotationValueMustBeConstant((TypeBinding)this.binding.declaringClass, this.name, expressions[i], false); 
            }  
        } else if (this.value.constant == Constant.NotAConstant) {
          if (valueType.isArrayType()) {
            scope.problemReporter().annotationValueMustBeArrayInitializer((TypeBinding)this.binding.declaringClass, this.name, this.value);
          } else {
            scope.problemReporter().annotationValueMustBeConstant((TypeBinding)this.binding.declaringClass, this.name, this.value, false);
          } 
        } 
        return;
      case 16:
        if (this.value instanceof ArrayInitializer) {
          ArrayInitializer initializer = (ArrayInitializer)this.value;
          Expression[] expressions = initializer.expressions;
          if (expressions != null)
            for (int i = 0, max = expressions.length; i < max; i++) {
              Expression currentExpression = expressions[i];
              if (!(currentExpression instanceof org.eclipse.jdt.internal.compiler.ast.ClassLiteralAccess))
                scope.problemReporter().annotationValueMustBeClassLiteral((TypeBinding)this.binding.declaringClass, this.name, currentExpression); 
            }  
        } else if (!(this.value instanceof org.eclipse.jdt.internal.compiler.ast.ClassLiteralAccess)) {
          scope.problemReporter().annotationValueMustBeClassLiteral((TypeBinding)this.binding.declaringClass, this.name, this.value);
        } 
        return;
    } 
    if (leafType.isEnum()) {
      if (this.value instanceof org.eclipse.jdt.internal.compiler.ast.NullLiteral) {
        scope.problemReporter().annotationValueMustBeConstant((TypeBinding)this.binding.declaringClass, this.name, this.value, true);
      } else if (this.value instanceof ArrayInitializer) {
        ArrayInitializer initializer = (ArrayInitializer)this.value;
        Expression[] expressions = initializer.expressions;
        if (expressions != null)
          for (int i = 0, max = expressions.length; i < max; i++) {
            Expression currentExpression = expressions[i];
            if (currentExpression instanceof org.eclipse.jdt.internal.compiler.ast.NullLiteral) {
              scope.problemReporter().annotationValueMustBeConstant((TypeBinding)this.binding.declaringClass, this.name, currentExpression, true);
            } else if (currentExpression instanceof NameReference) {
              NameReference nameReference = (NameReference)currentExpression;
              Binding nameReferenceBinding = nameReference.binding;
              if (nameReferenceBinding.kind() == 1) {
                FieldBinding fieldBinding = (FieldBinding)nameReferenceBinding;
                if (!fieldBinding.declaringClass.isEnum())
                  scope.problemReporter().annotationValueMustBeConstant((TypeBinding)this.binding.declaringClass, this.name, currentExpression, true); 
              } 
            } 
          }  
      } else if (this.value instanceof NameReference) {
        NameReference nameReference = (NameReference)this.value;
        Binding nameReferenceBinding = nameReference.binding;
        if (nameReferenceBinding.kind() == 1) {
          FieldBinding fieldBinding = (FieldBinding)nameReferenceBinding;
          if (!fieldBinding.declaringClass.isEnum())
            if (!fieldBinding.type.isArrayType()) {
              scope.problemReporter().annotationValueMustBeConstant((TypeBinding)this.binding.declaringClass, this.name, this.value, true);
            } else {
              scope.problemReporter().annotationValueMustBeArrayInitializer((TypeBinding)this.binding.declaringClass, this.name, this.value);
            }  
        } 
      } else {
        scope.problemReporter().annotationValueMustBeConstant((TypeBinding)this.binding.declaringClass, this.name, this.value, true);
      } 
    } else if (leafType.isAnnotationType()) {
      if (!valueType.leafComponentType().isAnnotationType()) {
        scope.problemReporter().annotationValueMustBeAnnotation((TypeBinding)this.binding.declaringClass, this.name, this.value, leafType);
      } else if (this.value instanceof ArrayInitializer) {
        ArrayInitializer initializer = (ArrayInitializer)this.value;
        Expression[] expressions = initializer.expressions;
        if (expressions != null)
          for (int i = 0, max = expressions.length; i < max; i++) {
            Expression currentExpression = expressions[i];
            if (currentExpression instanceof org.eclipse.jdt.internal.compiler.ast.NullLiteral || !(currentExpression instanceof org.eclipse.jdt.internal.compiler.ast.Annotation))
              scope.problemReporter().annotationValueMustBeAnnotation((TypeBinding)this.binding.declaringClass, this.name, currentExpression, leafType); 
          }  
      } else if (!(this.value instanceof org.eclipse.jdt.internal.compiler.ast.Annotation)) {
        scope.problemReporter().annotationValueMustBeAnnotation((TypeBinding)this.binding.declaringClass, this.name, this.value, leafType);
      } 
    } 
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope) && 
      this.value != null)
      this.value.traverse(visitor, scope); 
    visitor.endVisit(this, scope);
  }
  
  public void traverse(ASTVisitor visitor, ClassScope scope) {
    if (visitor.visit(this, scope) && 
      this.value != null)
      this.value.traverse(visitor, scope); 
    visitor.endVisit(this, scope);
  }
}
